% +--------------------------------+
% | Algorithm: FQR_POS_B version 1 |
% | Jos Antonio Apolinrio Jr.    |
% +--------------------------------+
clear; clc; close all; pack;
K         = 1000;      % number of iterations
ENS       = 10;        % ensemble
varx      = 1;         % input signal variance
varn      = 1e-8;      % noise variance
nrcoef    = 4;         % number of coeficients (N+1)
N         = nrcoef-1;  % filter order
num       = [1];       % num and den used to generate colored input signal
den       = [1 1.2 0.81];   
lambda    = 0.99;      % forgetting factor
lambda05  = sqrt(lambda);
lambdam05 = 1/lambda05;
Wopt      = [1 0.9*sqrt(-1) 0.1 0.2*sqrt(-1)]';              
MSE       = zeros(K,1);
delta     = 1e-6;
for ens=1:ENS,
  disp(ens);
  % Generating input and reference (desired) signals:
  x     = randn(K,1)+sqrt(-1)*randn(K,1);                             
  x     = filter(num,den,x); x=x/std(x);  x=x*sqrt(varx);
  noise = randn(K,1); noise=noise/std(noise); noise=noise*sqrt(varn);  
  d     = filter(conj(Wopt),1,x)+noise;
  %----------------
  % Initialization:
  %----------------  
  moduloef=delta;
  dfq2=zeros(N+1,1);
  dq2=zeros(N+1,1);
  ctheta=ones(N+1,1);
  stheta=zeros(N+1,1);
  f=zeros(N+1,1);
  %-------------------
  % Runing the filter:
  %-------------------
  for k=1:K 
    if rem(k,10000)==0
      disp(k);
    end      
    %-----------------------------------
    % 1. Obtaining dfq2(k+1) e efq1(k+1:
    %-----------------------------------
    efq1=conj(x(k));
    for i=1:N+1
        oldfriend=efq1;
        efq1=ctheta(i-1+1)*efq1-conj(stheta(i-1+1))*lambda05*dfq2(N+2-i);
        dfq2(N+2-i)=stheta(i-1+1)*oldfriend+ctheta(i-1+1)*lambda05*dfq2(N+2-i);
    end; % for i
    %--------------------------
    % 2. Obtaining ||ef(k+1)||:
    %--------------------------
    moduloef=sqrt(abs(efq1)^2+lambda*moduloef^2);
    %---------------------------
    % 3. Obtaining Qteta'f(k+1):
    %---------------------------
    moduloef0=moduloef;
    for i=1:N+1,
        oldfriend=moduloef0;
        moduloef0=sqrt(moduloef0^2+abs(dfq2(i))^2);
        cthetaprimef(N+1-i+1)=oldfriend/moduloef0;     
        sthetaprimef(N+1-i+1)=conj(dfq2(i))/moduloef0;      
    end; % for i
    %---------------------
    % 4. Obtaining f(k+1):
    %---------------------
    aux=x(k)/moduloef0;
    fold=f;
    f(N+1)=aux;
    for i=1:N
        f(N+1-i)=(fold(N+2-i)-conj(sthetaprimef(i-1+1))*aux)/cthetaprimef(i-1+1);
        aux=-sthetaprimef(i-1+1)*f(N+1-i)+cthetaprimef(i-1+1)*aux;
    end % for i
    %-------------------------
    % 5. Obtaining Qtheta(K+1)
    %-------------------------
    gamma=1;
    for i=1:N+1
        stheta(i-1+1)=f(N+2-i)/gamma;
        ctheta(i-1+1)=sqrt(1-abs(stheta(i-1+1))^2) ;
        gamma=ctheta(i-1+1)*gamma;
    end; % for i
    %-----------------------------
    % 6. Joint Process Estimation:
    %-----------------------------
    eq1=conj(d(k));
    for i=1:N+1
        oldfriend=eq1;
        eq1=ctheta(i-1+1)*eq1-conj(stheta(i-1+1))*lambda05*dq2(N+2-i);
        dq2(N+2-i)=stheta(i-1+1)*oldfriend+ctheta(i-1+1)*lambda05*dq2(N+2-i);
    end; % for i
    %------------------------------
    % 7. Updating the output error:
    %------------------------------
    ekaposteriori=conj(eq1)*gamma;
    ekapriori=conj(eq1)/gamma;
    MSE(k)=MSE(k)+abs(ekapriori)^2;
    %MSE(k)=MSE(k)+abs(ekaposteriori)^2;
  end % for k
end % for ens
MSE=MSE/ENS;
%-------------------
% Simulation result:
%-------------------
E=10*log10(MSE);
k=1:K;
plot(k,E,'k');
xlabel('k'); ylabel('MSE_{dB}');
title('FQRPOSB Version 1: learning curve');
%--------------------> END OF FQRPOSB1.M <--------------------%
