% +--------------------------------+
% | Algorithm: FQR_POS_B lattice   |
% | Jos Antonio Apolinrio Jr.    |
% +--------------------------------+
clear; clc; close all; pack;
K         = 1000;      % number of iterations
ENS       = 10;        % ensemble
varx      = 1;         % input signal variance
varn      = 1e-8;      % noise variance
nrcoef    = 4;         % number of coeficients (N+1)
N         = nrcoef-1;  % filter order
num       = [1];       % num and den used to generate colored input signal
den       = [1 1.2 0.81];   
lambda    = 0.99;      % forgetting factor
lambda05  = sqrt(lambda);
Wopt      = [1 0.9 0.1 0.2]';              
MSE       = zeros(K,1);
for ens=1:ENS
    disp(ens);
    % Generating input and reference (desired) signals:
    x     = randn(K,1);                              
    x     = filter(num,den,x); x=x/std(x);  x=x*sqrt(varx);
    noise = randn(K,1); noise=noise/std(noise); noise=noise*sqrt(varn);  
    d     = filter(Wopt,1,x)+noise;
    %----------------
    % Initialization:
    %---------------- 
    epsilon=lambda^(N+1)*1e-5; % sug. Theo
    moduloef=epsilon*ones(N+2,1);	
    fk=zeros(N+2,1);
    dfq2=zeros(N+1,1);
    dq2=zeros(N+1,1);
    ctheta=ones(N+1,1);
    stheta=zeros(N+1,1);
    %--------------------
    % Running the filter:
    %--------------------
    for k=1:K
        if rem(k,10000)==0
            disp(k);
        end
        % Feeding the first section:
        efq1(0+1)=x(k);
        moduloef(0+1)=sqrt(lambda*(moduloef(0+1))^2+(efq1(0+1))^2);
        aux=x(k)/moduloef(0+1);
        fkplus1(N+1+1)=aux;
        gamma(0+1)=1;
        eq1(0+1)=d(k);
        %---------------------------------------
        % N+1 SECTIONS OF THE LATTICE STRUCTURE:
        %---------------------------------------
        for i=1:N+1
            oldfriend=dfq2(N+2-i);
            dfq2(N+2-i)=stheta(i)*efq1(i-1+1)+ctheta(i)*lambda05*oldfriend;
            efq1(i+1)=ctheta(i)*efq1(i-1+1)-stheta(i)*lambda05*oldfriend;
            moduloef(i+1)=sqrt(efq1(i+1)^2+lambda*moduloef(i+1)^2);
            cthetaprimef(i)=moduloef(i+1)/moduloef(i-1+1);
            sthetaprimef(i)=dfq2(N+2-i)/moduloef(i-1+1);
            fkplus1(N+1-i+1)=(fk(N+2-i+1)-sthetaprimef(i)*aux)/cthetaprimef(i);
            aux=-sthetaprimef(i)*fkplus1(N+1-i+1)+cthetaprimef(i)*aux;
            gamma(i+1)=sqrt((gamma(i-1+1))^2-(fkplus1(N+2-i+1))^2);
            ctheta(i)=gamma(i+1)/gamma(i-1+1);
            stheta(i)=fkplus1(N+2-i+1)/gamma(i-1+1);
            oldfriend=dq2(N+2-i);
            dq2(N+2-i)=stheta(i)*eq1(i-1+1)+ctheta(i)*lambda05*oldfriend;
            eq1(i+1)=ctheta(i)*eq1(i-1+1)-stheta(i)*lambda05*oldfriend;
        end; % for i
        fk=fkplus1;
        MSE(k)=MSE(k)+(eq1(N+1+1)/gamma(N+1+1))^2;
    end % for k
end % for ens
MSE=MSE/ENS;
%-------------------
% Simulation result:
%-------------------
E=10*log10(MSE);
k=1:K;
plot(k,E,'k');
xlabel('k'); ylabel('dB');
title('FAST QRD A POSTERIORI (LOWER TRIANG.) LATTICE: MSE dB');
%-------------------> END OF FQR2POSL.M <-------------------%
