function [E, stheta, ctheta, sintheta_prime_f, costheta_prime_f, dq2, dfq2, Ef, a, gamma] = ...
block_mcfqr_pri_b (stheta, ctheta, sintheta_prime_f, costheta_prime_f, dq2, dfq2, Ef, a, gamma, X, d, lambda, M, L, pi)
%  +---------------------------------------+
%  |     Block Multichannel FQRD-RLS       |
%  |    Algorithm  (a priori version)      |
%  |      First version:    2005 by        |
%  |                                       |
%  |    Apolinário Jr., apolin@ime.eb.br   |
%  |               &                       |
%  |     Werner, stefan.werner@hut.fi      |
%  |               &                       |
%  |    Ramos, alopesramos@yahoo.com.br    |
%  |                                       |
%  |     Last update: Summer 2008 by       |
%  |         Antonio L. L. Ramos           |
%  |       alopesramos@yahoo.com.br        |
%  +---------------------------------------+
% The algorithms have been implemented in a simple way
% (if I can say so...) to make them easy to understand.
% That means that, after you have mastered them, there is a
% lot of simplifications that can be achieved.
% So, feel free to modify and/or adjust them to your own experiments.
%
% It should be clear that the files are meant to be used for educational
% purposes only. No warranty for comercial use is offered and neither the publisher nor
% the authors shall be liable for any loss of any type.


sinBar_f=zeros(1:M);
cosBar_f=zeros(1:M);
    %-------------------------------------+
    % 1 Obtaining Dfq2(k+1) and efq1(k+1):|
    %-------------------------------------+    
    efq1(1,:) = X(1,:);
    for j = 1:L
        for s = 1:M
            temp1 = efq1(s);
            temp2 = lambda^.5 * dfq2 (L - j + 1, s);
            efq1(s) = temp1 * ctheta(j) - temp2 * stheta(j);
            dfq2(L - j + 1, s) = temp1 * stheta(j) + temp2 * ctheta(j);
        end
    end 
    %-----------------------+
    % 2  Obtaining Ef(K+1): |
    %-----------------------+
    temp_efq1 = efq1;
    for i = 1:M
        temp1 = temp_efq1(i);
        temp2 = lambda^.5 * Ef(M - i + 1, i);
        temp3 = sqrt (temp1^2 + temp2^2);
        sinBar_f (i) = temp1/temp3;
        cosBar_f (i) = temp2/temp3;
        Ef (M - i + 1, i) = temp3;
        for s = i + 1:M
            temp1 = temp_efq1(s);
            temp2 = lambda^.5 * Ef (M - i + 1, s);
            temp_efq1(s) = temp1 * cosBar_f(i) - temp2 * sinBar_f(i);
            Ef (M - i + 1, s) = temp1 * sinBar_f(i) + temp2 * cosBar_f(i);
        end
    end
    %--------------------+
    % 3 Obtaining r(k+1):|
    %--------------------+    
    % r = inv(Ef)' * efq1' / (lambda^.5 * gamma); % To be avoided    
    temp_gamma = 1/gamma;
    for i = 1:M
        r(M - i + 1, 1) = temp_gamma * sinBar_f(i) / cosBar_f(i);
        temp_gamma = temp_gamma * cosBar_f(i) + r(M - i + 1, 1) * sinBar_f(i);
    end
    %---------------------+
    % 4 Obtaining a(P+M)(k+1):|
    %---------------------+    
    for j = 1:L
        for s = 1:M
            if j<=(L-pi(s)+s)
                temp1 = a (M + j);
                temp2 = r(M - s + 1);
                a(j) = temp1 * costheta_prime_f (j, s) - temp2 * sintheta_prime_f (j, s);
                a(M + j) = a(j);
                r(M - s + 1) = temp1 * sintheta_prime_f (j, s) + temp2 * costheta_prime_f (j, s);
            end
        end
	end
    a(L+1:L+M) = r;   
    %----------------------------------------------------------------------
    % Performing the desired permutations
    % (implementing a=Pbar*a;)
    for s = M:-1:2 % 1 % According to this formulation, channel 1 will
         aux=a(L+M-s+1);  % always be in the lead and... P1=I
         a(L+M-pi(s)+2:L+M-s+1)=a(L+M-pi(s)+1:L+M-s);
         a(L+M-pi(s)+1)=aux;
    end
    
% pause,keyboard
    %--------------------------+
    % 5 Obtaining Q'theta_f(k+1):|
    %--------------------------+    
    Ef_0 = Ef;
    temp_dfq2 = dfq2;
    for j = 1:L
        for s = 1:M
            if j<=(L-pi(s)+s) % Only some entries in dfq2 are anihilated               
                temp1 = temp_dfq2 (j, s);
                temp2 = Ef_0 (M - s + 1, s);
                sintheta_prime_f (j, s) = temp1 / sqrt (temp1^2 + temp2^2);
                costheta_prime_f (j, s) = temp2 / sqrt (temp1^2 + temp2^2);
                Ef_0 (M - s + 1, s)  = sqrt (temp1^2 + temp2^2);
                for p = s + 1:M
                    temp1 = temp_dfq2 (j, p);
                    temp2 = Ef_0 (M - s + 1, p);
                    temp_dfq2(j,p)=temp1*costheta_prime_f(j, s) - temp2 * sintheta_prime_f(j, s);
                    Ef_0(M-s+1,p) =temp1*sintheta_prime_f(j, s) + temp2 * costheta_prime_f(j, s);
                end
            end
        end
    end
    %----------------------------------------+
    % 6 Obtaining Qtheta(k+1) and gamma(k+1):|
    %----------------------------------------+    
    inv_gamma = 1;
    for j = 1:L    
        temp1 = inv_gamma;
        temp2 = a(L + M - j + 1);
        temp3 = sqrt (temp1^2 + temp2^2);
        stheta(j) = temp2 / temp3;
        ctheta(j) = temp1 / temp3;
        inv_gamma = temp1 * ctheta(j) + temp2 * stheta(j);
    end
    gamma = 1/inv_gamma;
    %-------------------+
    % 7 Join Estimation:|
    %-------------------+    
%     eq1 = d(k);
    eq1 = d;
    for j = 1:L
        temp1 = eq1;
        temp2 = lambda^.5 * dq2(L - j + 1);
        eq1 = temp1 * ctheta (j) - temp2 * stheta(j);
        dq2(L - j + 1) = temp1 * stheta (j) + temp2 * ctheta(j);
    end
	E = eq1 / (realmin + gamma); % a priori error
% 	E = eq1 * gamma; % a posteriori error     
