    function [E, steta, cteta, stetalinhaf, ctetalinhaf, dq2, dfq2, ef, f, gamma] = ...
seq_mcfqr_pos_b (steta, cteta, stetalinhaf, ctetalinhaf, dq2, dfq2, ef, f, gamma, X, d, lambda, M, L, pi)
%  +--------------------------------------+
%  |   Sequential Multichannel FQRD-RLS   |
%  |  Algorithm  (a posteriori version)   |
%  |    First version  Spring 2004 by:    |
%  |     Last update: Summer 2008 by:     |
%  |         Antonio L. L. Ramos          |
%  |       alopesramos@yahoo.com.br       |
%  +--------------------------------------+
% The algorithms have been implemented in a simple way
% (if I can say so...) to make them easy to understand.
% That means that, after you have mastered them, there is a
% lot of simplifications that can be achieved.
% So, feel free to modify and/or adjust them to your own experiments.
%
% It should be clear that the files are meant to be used for educational
% purposes only. No warranty for comercial use is offered and neither the publisher nor
% the authors shall be liable for any loss of any type.

efq1(1, :) = (X(1,:));
eq1 = d;
for i = 1:M
	% Step 1: Updating dfq2(k-1)
    %------------------------------------+
    % 1. Obtaining dfq2(j ,i) e efq1(1, i):|
    %------------------------------------+
	for j = 1:L % 1:k
      oldfriend     = efq1(1, i);
      efq1(1, i)    = cteta(j)*efq1(1, i) + steta(j)*lambda^0.5*dfq2(L-j+1, i);
      dfq2(L-j+1, i)= cteta(j)*lambda^0.5*dfq2(L-j+1, i) - conj(steta(j))*oldfriend;
  end;
   	%--------------------------+
	% 2. Obtaining ||ef(k+1)||:|
	%--------------------------+
    ef(i)   = sqrt( abs(efq1(i))^2 + lambda*abs(ef(i))^2);
	%--------------------------+
	% 3. Obtaining Qteta'f(k+1): |
	%--------------------------+
    % Obtaining Qthetaf'(k+1)
	efi_0 = ef(i);
    for j = L:-1:pi(i),
      temp = efi_0;
      efi_0 = sqrt(efi_0^2 + abs(dfq2( L-j+1, i))^2);
      ctetalinhaf(j, i) = abs(temp/ efi_0);
      stetalinhaf(j, i) = conj(ctetalinhaf(j, i)*dfq2(L-j+1, i)/temp);
    end; 
    %--------------------+
	% 4. Obtaining f(k+1): |
	%--------------------+
    p = gamma*conj(efq1(i))/ ef(i);
    for j = L:-1:pi(i),
        temp2       = f(L-j+1+1);
        f(L-j+1)    = ctetalinhaf(j, i)*temp2 - conj(stetalinhaf(j, i))*p;
        p           = stetalinhaf(j, i)*temp2 + ctetalinhaf(j, i)*p;
    end  
    f(L+1 - pi(i)+1) = p;
    %-----------------------+
	% 5. Obtaining Qteta(K+1) |
	%-----------------------+
    if (i == 1)
        gamma = 1;
    else
        gamma = gamma_old;
    end
    for j = pi(i):L
        steta(j)   = -conj( f( L -j+1  +1))/gamma;
        cteta(j)   = sqrt(1 - abs(steta(j))^2 );
        gamma      = cteta(j)*gamma;
        if (j ==  pi(i+1)-1)
            gamma_old = gamma;
        end
    end
end % i loop
%------------------------------+
% 6. Joint Process Estimation: |
%------------------------------+
% eq1 = d;
for j = 1:L
  oldfriend = eq1;
  eq1       = cteta(j)*eq1+steta(j)*lambda^0.5*dq2(L-j+1);
  dq2(L-j+1)= cteta(j)*lambda^0.5*dq2(L-j+1) - conj(steta(j))*oldfriend;
end
%-------------------------------+
% 7. Updating the output error: |
%-------------------------------+
E = eq1 / (realmin + gamma);    % a priori error
% E = eq1*gamma;                % a posteriori error
