function [e,eh]=hrls(y,x,h,l,M);
%
% function [e,eh]=hrls(y,x,h,l,M)
%
% The Householder RLS algorithm
%
% y: channel output vector
% x: channel input vector
% h: channel impulse response vector
% l: forgetting factor
% M: size of data 
%
% e: squared a priori error vector
% eh: squared weight error vector

%
%Initialization
%
N = length(h);
delta=1*l^N;
theta = zeros(N,1);
S=inv(sqrt(delta))*eye(N);
e=[];
eh = [];
w = zeros(N,M);
%
% The algorithm
%
for m=N:M+N-1
   u=[x(m:-1:m-N+1)];   
   v=sqrt(1/l)*S'*u.';
   er=y(m-N+1)-theta'*u.';
   d = sqrt(1+norm(v)^2);
   z = -(S*v)/(sqrt(l)*d);
   S = sqrt(1/l)*S + (z*v')/(1+d);
   theta=theta-(conj(er)/d)*z;
   eh = [eh norm(theta-h')^2];
   e = [e er'*er];
end
figure
semilogy(eh,'g')