function e = sw_qrd_rls(y,x,h,L,M,Q);
%
% function e = sw_qrd_rls(y,x,h,L,M,Q);
%
% The sliding window Householder QRD-RLS algorithm
%
% y: channel output vector
% x: channel input vector
% h: channel impulse response vector
% L: the sliding window size
% M: size of data 
% Q: the block size
%
% e: squared weight error vector

%
% Initialization
%
N = length(h);
U=eye(N);
dq = zeros(N,1);
e=[];
X = zeros(M,N);
d = zeros(M,1);
Y = zeros(N+Q,N);
ww = zeros(N,1);
%
% Construction of data matrices
%
for m=N:M+N-1
 u=[x(m:-1:m-N+1)];
 X(M+N-m,:) = u;
 d(M+N-m) =  y(m-N+1);
end;
%
% Execution of L steps of the Householder inverse QRD-RLS algorithm
%
w = zeros(N,M/Q);
for i=1:L,
    Xt = X(M-i*Q+1:M-(i-1)*Q,:);
    dt = d(M-i*Q+1:M-(i-1)*Q);
    G = -U*Xt.';
    Ga = [eye(Q);G];
    Y = [zeros(Q,N); U];
    for k=1:N,
        [Z,H] = roht(Ga,Q+k,1);
        Y = H*Y;
        Ga = Z;
    end;
    U = Y(Q+1:Q+N,:);
    D = Ga(1:Q,:);
    E = Y(1:Q,:);
    ww = ww -E'*inv(D')*(conj(dt) - conj(Xt)*ww);
    w(:,i)= ww;
end;
%
% The algorithm
%
phi = [-ones(1,Q), ones(1,N)];
for i = L+1:M/Q,
    % Update step
    Xt = X(M-i*Q+1:M-(i-1)*Q,:);
    dt = d(M-i*Q+1:M-(i-1)*Q);
    G = -U*Xt.';
    Ga = [eye(Q);G];
    Y = [zeros(Q,N); U];
    for k=1:N,
        [Z,H] = roht(Ga,Q+k,1);
        Y = H*Y;
        Ga = Z;
    end;
    U = Y(Q+1:Q+N,:);
    D = Ga(1:Q,:);
    E = Y(1:Q,:);
    ww = ww-E'*inv(D')*(conj(dt) - conj(Xt)*ww);
    
    % Downdate step 
    Xt = X(M-(i-L)*Q+1:M-((i-L)-1)*Q,:);
    dt = d(M-(i-L)*Q+1:M-((i-L)-1)*Q);
    G = -U*Xt.';
    Ga = [eye(Q);G];
    Y = [zeros(Q,N); U];
    for k=1:N,
        [Z,H] = rhht(Ga,phi,Q+k,1);
        Y = H*Y;
        Ga = Z;
    end;
    U = Y(Q+1:Q+N,:);
    D = Ga(1:Q,:);
    E = Y(1:Q,:);
    ww = ww -E'*inv(D')*(conj(dt) - conj(Xt)*ww);
    w(:,i)= ww;
end;    
    
for i=1:M/Q,
    e(i) = norm(w(:,i)-h')^2;
end;
figure
semilogy(e)